#include "CTR_Fio.h"
#include <nn/fs.h>
#include <string.h> // str***
#include <stdlib.h> // mbstowcs
#include "CTR_Define.h" // memset

using namespace nn::fs;
#ifdef __cplusplus
extern "C" {
#endif


#define 	 MAX_DSC_NUM 10
FileReader gReader[MAX_DSC_NUM];
int gDscCnt = 0;


/**********************************************************
*  name    : CTR_FIO_FileInit()
* param  : None.
*  return  : None.
* summary: Initialize the file system
***********************************************************/
void CTR_FIO_FileInit()
{
    gDscCnt = 0;
    return;
}
/**********************************************************
*  name    : CTR_FIO_FileTerm()
* param  : None.
*  return  : None.
* summary: None.
***********************************************************/
void CTR_FIO_FileTerm( void )
{
    return;
}
/**********************************************************
*  name    : CTR_FIO_FileOpen()
* param  : Filename to open.
*        : Mode (Not yet used)
*  return  : File information 
* summary: Opens a file and returns the address to that file's information.
*          
***********************************************************/
FILE *CTR_FIO_FileOpen(const char *path, const char *mode)
{
	FILE  *file=NULL;
	nn::Result  result;
    wchar_t wPath[256];
    wchar_t wBuf[256];
	int     len=0;

	MEMSET( wPath,0,sizeof(wPath)/sizeof(wPath[0]));
	MEMSET( wBuf,0,sizeof(wBuf)/sizeof(wBuf[0]));

	if( path == NULL ){
		NN_LOG( "[Error] path = NULL \n");
		return NULL;
	}

    mode = mode; //warning

    if( gDscCnt >= MAX_DSC_NUM -1 ){
		NN_LOG("[Error] Max DSC num \n" );
		return NULL;
	}	

	if( path == NULL ){
		NN_LOG("[Error] path is NULL \n" );
		return NULL;
	}

	NN_LOG( "fname=%s\n", path );
	len=strlen( path );

	// char -> wchar_t
	mbstowcs( wPath, path, len );


	result = gReader[gDscCnt].TryInitialize( wPath );
    if( result.IsFailure()){
        NN_LOG("File Open Error\n");
		return NULL;
	}else{
        NN_LOG("File Open Success\n");
	}

	// Dummy to match other platforms and I/F
	file = (FILE*)&gReader[gDscCnt];
	gDscCnt++;

	return file;
}

/**********************************************************
*  name    : CTR_FIO_FileClose()
* param  : File stream to close
*  return  : 0: success
           Failed EOF
* summary: Closes files that were opened
***********************************************************/
int CTR_FIO_FileClose( void *stream )
{
	int i=0;
    FileReader  *local = (FileReader*)stream;
	for( i=0;i< MAX_DSC_NUM;i++){
        if( &gReader[i] == local ){
        	local->Finalize();
        	gDscCnt--;
			break;
		}
	}
    return 0;
}



/**********************************************************
*  name    : CTR_FIO_GetFileSize()
* param  : Gets the file stream size
*  return  : File size.
* summary: Gets the size of the file that is already open
***********************************************************/
unsigned long CTR_FIO_GetFileSize( void *stream )
{
    u32 size=0;
    FileReader  *local = (FileReader*)stream;

	size = (u32)local->GetSize();
    return (unsigned long)size;
}



/**********************************************************
*  name    : CTR_FIO_FileRead()
* param  : Size to read
*  return  : Size that was read.
* summary: Reads a portion of the specified size from the opened file
***********************************************************/
long CTR_FIO_FileRead( void* ptr, unsigned long elsize, unsigned long elcnt, void *stream )
{
    s32 ret=-1;
    FileReader  *local = (FileReader*)stream;

   	ret = local->Read( ptr, (size_t)( elsize*elcnt ) );

    return ret;
}



/**********************************************************
*  name    : CTR_FIO_FGets()
* param  : Pointer to the buffer region
* param  : Buffer size
* param  : Pointer to the file descriptor
*  return  : Pointer to the read buffer
* summary: Replaces fgets()
***********************************************************/
char *CTR_FIO_FGets(char *buf, int len, void *stream)
{
    s32 ret=0;
	int i;
    u32 work_size = 0;
    FileReader  *local = (FileReader*)stream;

    static s32 readed_size = 0;

	work_size = (u32)local->GetSize();
	if( readed_size >= work_size ){
		readed_size = 0;
		return NULL;
	}
	
	if( (work_size - readed_size ) < len ){
		len = (work_size - readed_size ) + 1;
	}


	for( i=0;i< MAX_DSC_NUM;i++){
        if( &gReader[i] == local ){
			break;
		}
	}
	if( i==MAX_DSC_NUM ) return NULL;

    ret = local->Read( buf, len );
    if (ret <= 0){ 
        /* Returns NULL even for file's terminator */
        return NULL;
    } else {
        for (i = 0; i < ret; i++) {
            if( buf[i]=='\n' ) {
                ++i;
                break;
            } 
        }
		buf[i] = '\0';
        (void)local->Seek( (s64)(i - ret), POSITION_BASE_CURRENT );

		readed_size +=i;
    }

    return buf;

}

/**********************************************************
*  name    : CTR_FIO_FSeek()
* param  : Pointer to the file descriptor
* param  : Offset to seek
* param  : Seek direction
*  return  :  0: Success
*  return  : -1: Failed
* summary:
***********************************************************/
int CTR_FIO_FSeek( void *stream, long offset, int dir )
{
	PositionBase base;
    FileReader  *local = (FileReader*)stream;

	if( dir == SEEK_SET ){
        base = POSITION_BASE_BEGIN;
	} else if( dir == SEEK_END ){
		base = POSITION_BASE_END;
	}else{ // SEEK_CUR
		base = POSITION_BASE_CURRENT;
	}

    local->Seek( (s64)offset, base );

    return 0;
}

/**********************************************************
*  name    : CTR_FIO_GetFileNames()
* param  : File streaming
*          Store destination structure
*  return  : Number of loaded file names
* summary: Stores the file names in the specified file stream in the textfile structure
***********************************************************/
int CTR_FIO_GetFileNames( void* fp, textfile *file)
{
	// warning
	fp=fp;
	file=file;
    return 0;
}

#ifdef __cplusplus
}
#endif



