/*-----------------------------------------------------------------------*
 Project: [Common]
 File: [CTR_Memory.cpp]

 Copyright (C) 2009-2014 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law.  They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
*-----------------------------------------------------------------------*/
/******************************************************************
                        include files
*******************************************************************/
#include "CTR_Memory.h"
#include "CTR_Define.h"
#include <nn/fnd.h>
#include <string.h> // memset
using namespace nn::fnd;

#ifdef __cplusplus
extern "C" {
#endif


/******************************************************************
            static declaration
*******************************************************************/
/* ExpHeap for app. */
static ExpHeap gAppHeap;
static int gAllocCount=0;

/******************************************************************
            Functions Available Externally
*******************************************************************/
/*
 
 
 
           
 
 
*/
int CTR_MEM_Init(void)
{
    gAllocCount=0;
    // Create heap

    const s32 nDeviceMemorySizeOther = 32 * 1024 * 1024;
    gAppHeap.Initialize(nn::os::GetDeviceMemoryAddress(), nDeviceMemorySizeOther, nn::os::ALLOCATE_OPTION_LINEAR);

    PRINT_OUT("[Heap Rest = %ld allocCnt=%d @Init()\n", CTR_MEM_GetFreeSizeOfHeap(), gAllocCount);
 
    return 0;
}

/*
 
 
 
 
*/
void CTR_MEM_Term(void)
{
    PRINT_OUT("[Heap Rest = %ld allocCnt=%d @Term()\n", CTR_MEM_GetFreeSizeOfHeap(), gAllocCount);
    gAppHeap.Finalize();
    return ;
}

/*
 
 
 
           
 
 
*/
void *CTR_MEM_Malloc32( size_t size )
{
    void *allocator = NULL;
    allocator = (gAppHeap.Allocate( size, 32 ));
	if( allocator == NULL ){
        PRINT_OUT(" [ERROR] Alloc Failed\n");
		return NULL;
	}
    MEMSET( allocator, 0, size );
    gAllocCount++;
#ifdef MEMORY_DEBUG
    PRINT_OUT("   Alloc Size=%d rest=%ld count=%d (0x%08x)\n", size, CTR_MEM_GetFreeSizeOfHeap(),gAllocCount, allocator);
#endif /* #ifdef MEMORY_DEBUG */
    return allocator;
}


/*
 
 
 
           
 
 
*/
void *CTR_MEM_Malloc( size_t size )
{
    void *allocator=NULL;
    allocator = (gAppHeap.Allocate(size));
	if( allocator == NULL ){
        PRINT_OUT(" [ERROR] Alloc Failed\n");
		return NULL;
	}
    MEMSET( allocator, 0, size );
    gAllocCount++;
#ifdef MEMORY_DEBUG
    PRINT_OUT("   Alloc Size=%d rest=%ld count=%d (0x%08x)\n", size, CTR_MEM_GetFreeSizeOfHeap(),gAllocCount, allocator);
#endif /* #ifdef MEMORY_DEBUG */
    return allocator;

}

/*
 
 
 
 
           
 
 
           
           
*/
void *CTR_MEM_MallocEx( size_t size, int alignment )
{
    void *allocator=NULL;
    allocator = (gAppHeap.Allocate(size, alignment));
	if( allocator == NULL ){
        PRINT_OUT(" [ERROR] Alloc Failed\n");
		return NULL;
	}
    MEMSET( allocator, 0, size );
    gAllocCount++;
#ifdef MEMORY_DEBUG
    PRINT_OUT("   Alloc Size=%d align(%d) rest=%ld count=%d (0x%08x)\n", size, alignment, CTR_MEM_GetFreeSizeOfHeap(),gAllocCount, allocator);
#endif /* #ifdef MEMORY_DEBUG */
    return allocator;

}

/*
 
 
 
 
*/
void CTR_MEM_Free( void *ptr )
{
    gAppHeap.Free( ptr );
    gAllocCount--;

#ifdef MEMORY_DEBUG
    PRINT_OUT("   Free() rest=%ld count=%d (0x%08x)\n", CTR_MEM_GetFreeSizeOfHeap(),gAllocCount,ptr);
#endif /* #ifdef MEMORY_DEBUG */

    return;
}


/*
 
 
 
 
*/
unsigned long CTR_MEM_GetFreeSizeOfHeap( void )
{
    return (unsigned long)( gAppHeap.GetTotalFreeSize() );
}


/*
 
 
 
 
 
 
*/
void *CTR_MEMSET( void* s, int c, size_t n )
{
    std::memset( s,c,n );
    return s;
}


/*
 
 
 
 
 
 
*/
void CTR_MEMCPY( void* dst, const void* src, size_t size )
{
    std::memcpy( dst, src, size );
    return;
}

/*
 
 
 
 
 
           
 
*/
int CTR_MEMCMP( const void* mem1, const void* mem2, size_t size )
{
   return std::memcmp( mem1, mem2, size );
}

#ifdef __cplusplus
}
#endif



