// ----------------------------------------------------------------------
// Detection Results Display Window Class
// ----------------------------------------------------------------------

// ----------------------------------------------------------------------
// include system header
// ----------------------------------------------------------------------
#include	<nn.h>
#include	<nn/gx.h>
#include	<nn/math.h>
#include	<nn/os.h>
#include	<nn/hid.h>
#include	<nn/fnd.h>
#include	<nn/fs.h>

#include	<string.h>

// ----------------------------------------------------------------------
// include mgr header
// ----------------------------------------------------------------------
#include	"mgr.h"					// mgr header
using namespace mw::mgr;

// ----------------------------------------------------------------------
// include original header
// ----------------------------------------------------------------------
#include	"CTR_Define.h"
#include	"CtrlGraphics.h"
#include	"CtrlCamera.h"
#include	"TxtWindow.h"
#include	"TxtWinResultMgr.h"

// ----------------------------------------------------------------------
// CTxtWinResultMgr::CTxtWinResultMgr
// ----------------------------------------------------------------------
// Content
//   Constructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtWinResultMgr::CTxtWinResultMgr()
	: CTxtWindow("Result")
{

}

// ----------------------------------------------------------------------
// CTxtWinResultMgr::~CTxtWinResultMgr
// ----------------------------------------------------------------------
// Content
//   Destructor
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
CTxtWinResultMgr::~CTxtWinResultMgr()
{

}

// ----------------------------------------------------------------------
// CTxtWinResultMgr::Draw
// ----------------------------------------------------------------------
// Content
//   Displays Window.
// ----------------------------------------------------------------------
// Arguments
//   graphics		[ I ] Specify graphics class.
//   display		[ I ] Specify Window show display.
//   sx				[ I ] Specify Window display x-coordinate.
//   sy				[ I ] Specify Window display y-coordinate.
// ----------------------------------------------------------------------
// Return value
//   Always returns 0.
// ----------------------------------------------------------------------
int CTxtWinResultMgr::Draw(CCtrlGraphics* graphics, const s32 display, int sx, int sy)
{
	char		str[64];
	int			n			= 0;
	int			body_num	= m_result.bodynum;
	MGObject*	object		= m_result.mgobject;
	int			lines		= 12;

	if (body_num + 2 > lines)
	{
		lines = body_num + 2; 
	}

	CTxtWindow::SetLines(lines);
	
	sprintf(str, "bodynum:%d\n", body_num);
	CTxtWindow::SetText(0, str, 0.0, 0.0, 0.0);

	sprintf(str, "------------------------------\n");
	CTxtWindow::SetText(1, str, 1.0, 1.0, 1.0);

	for (n = 0 ; n < body_num ; n++)
	{
		sprintf(str, "%d: [%02d] (%3d,%3d)-(%3d,%3d) (%3d,%3d)\n",
			n + 1,
			(s32)object->id,
			object->rect.sx,
			object->rect.sy,
			object->rect.ex,
			object->rect.ey,
			object->center.x,
			object->center.y);

		CTxtWindow::SetText(n + 2, str, 0.0, 0.0, 0.0);
		object++;
	}

	for (; n < 10 ; n++)
	{
		sprintf(str,"%d: [--] (---,---)-(---,---) (---,---)\n", n);
		CTxtWindow::SetText(n + 2, str, 0.0, 0.0, 0.0);
	}

	CTxtWindow::DrawWindow(graphics, display, sx, sy);

	return 0;
}

// ----------------------------------------------------------------------
// CTxtWinResultMgr::UpdateResult
// ----------------------------------------------------------------------
// Content
//   Updates the result value.
// ----------------------------------------------------------------------
// Arguments
//   None
// ----------------------------------------------------------------------
// Return value
//   None
// ----------------------------------------------------------------------
void CTxtWinResultMgr::UpdateResult(MGResult* result)
{
	memcpy(&m_result, result, sizeof(MGResult));
}
